from django.contrib import admin

from unchained_chat.inlines import MessageInline
from unchained_chat.models import ChatRoom

@admin.register(ChatRoom)
class ChatRoomAdmin(admin.ModelAdmin):
    inlines = [MessageInline]
    list_display = ['name']



from django.apps import apps
from django.contrib import admin
from .apps import DjAppConfig


models = apps.get_app_config(DjAppConfig.name).get_models()


# Keep this the last after all manual registeration
for model in models:
    try:
        admin.site.register(model)
    except admin.sites.AlreadyRegistered:
        pass



from fcm_django.api.rest_framework import DeviceViewSetMixin, FCMDeviceSerializer
from fcm_django.models import FCMDevice
from rest_framework.mixins import CreateModelMixin, DestroyModelMixin
from rest_framework.viewsets import ReadOnlyModelViewSet, GenericViewSet

from unchained_notification.models import Notification
from unchained_notification.serializers import NotificationSerializer


class NotificationViewSet(ReadOnlyModelViewSet):
    serializer_class = NotificationSerializer

    def get_queryset(self):
        return Notification.objects.filter(user=self.request.user)


class NoListingFCMDeviceViewSet(DeviceViewSetMixin, CreateModelMixin, DestroyModelMixin, GenericViewSet):
    queryset = FCMDevice.objects.filter(active=True)
    serializer_class = FCMDeviceSerializer






from django.contrib.auth import get_user_model
from rest_framework import status, generics
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.authtoken.models import Token
from social_core.backends import facebook
from google.oauth2 import id_token, service_account
from google.auth.transport import requests
from unchained_auth.serializers import GoogleUIDSerializer
from django.conf import settings

try:
    credentials = service_account.Credentials.from_service_account_file(settings.GOOGLE_APPLICATION_CREDENTIALS)
    credentials.with_scopes(['https://www.googleapis.com/auth/cloud-platform'])
except (TypeError, FileNotFoundError):
    pass


class SocialView(generics.CreateAPIView):
    serializer_class = GoogleUIDSerializer
    permission_classes = (AllowAny,)

    def create(self, request, *args, **kwargs):
        User = get_user_model()
        backend = kwargs.get('backend')
        serializer = self.get_serializer(data=request.data)
        assert serializer.is_valid(), "Invalid token"
        token = serializer.validated_data['token']
        user = None
        if backend == 'google':
            idinfo = id_token.verify_oauth2_token(token, requests.Request())
            assert idinfo.get('iss') in ['accounts.google.com', 'https://accounts.google.com'], 'Wrong issuer.'
            email = idinfo['email']
            user, _ = User.objects.get_or_create(email=email)
            user.set_unusable_password()
            user.last_name = idinfo['family_name']
            user.first_name = idinfo['given_name']
            user.save()
        elif backend == 'facebook':
            user = facebook.FacebookOAuth2().do_auth(access_token=token)

        assert user, 'Unauthorized'
        token, _ = Token.objects.get_or_create(user=user)
        return Response({
                'token': token.key
        }, status=status.HTTP_200_OK)





from rest_framework import serializers
from rest_framework.serializers import ModelSerializer

from unchained_chat.models import Message, ChatRoom


class MessageSerializer(ModelSerializer):
    class Meta:
        model = Message
        fields = '__all__'


class ChatRoomSerializer(ModelSerializer):
    messages = MessageSerializer(many=True)
    last_message = serializers.ReadOnlyField()

    class Meta:
        model = ChatRoom
        fields = '__all__'



from django.contrib.auth import get_user_model
from rest_framework import status
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.generics import CreateAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.authtoken.models import Token as Rest_Token
from unchained_auth.serializers import UserSerializer


class RegisterAPIView(CreateAPIView):
    permission_classes = (AllowAny,)
    serializer_class = UserSerializer

    def create(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data)
        assert serializer.is_valid(), str(serializer._errors)
        new_user = get_user_model()(
                email=request.data['email']
        )
        new_user.set_password(request.data['password'])
        new_user.save()
        return Response(status=status.HTTP_201_CREATED)


class LoginView(ObtainAuthToken):
    def post(self, request, *args, **kwargs):
        response = super(LoginView, self).post(request, *args, **kwargs)
        token = Rest_Token.objects.get(key=response.data['token'])

        return Response({
            'token': token.key
        }, status=status.HTTP_200_OK)


class LogoutView(APIView):
    def get(self, request, format=None):
        request.user.auth_token.delete()
        return Response(status=status.HTTP_200_OK)



from django.urls import path, include
from django.views.generic import TemplateView
from rest_framework import routers

from unchained_chat.views import ChatRoomViewSet


router = routers.DefaultRouter()
router.register(r'chatrooms', ChatRoomViewSet, basename='chatrooms')

urlpatterns = [
    path('test', TemplateView.as_view(template_name='templates/websocket_test.html')),
    path('', include(router.urls)),
]


# Generated by Django 3.1.7 on 2021-03-07 14:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unchained_auth', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='fullname',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



# Generated by Django 3.1.4 on 2020-12-29 17:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import unchained_utils.v0.base_classes
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.CharField(default=uuid.uuid4, editable=False, max_length=36, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True, db_index=True)),
                ('text', models.CharField(max_length=200)),
                ('title', models.CharField(max_length=200)),
                ('type', models.CharField(max_length=50)),
                ('payload', models.TextField(blank=True, null=True)),
                ('user', unchained_utils.v0.base_classes.LooseForeignKey(blank=True, db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'get_latest_by': 'created_at',
                'abstract': False,
            },
        ),
    ]



from unchained_auth.models import User
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin

from django.contrib.auth.models import Group


admin.site.unregister(Group)

@admin.register(User)
class UserAdmin(UserAdmin):
    list_display = ['__str__', 'username', 'email', 'phone']
    search_fields = ('id', 'username', 'email', 'phone')
    ordering = ('created_at',)
    date_hierarchy = 'created_at'





from django.urls import path, re_path
from django.conf import settings

urlpatterns = []

if settings.UNCHAINED_AUTH.get('PWD'):
    from unchained_auth.views.main import LoginView, LogoutView

    urlpatterns.extend([
        path('login/', LoginView.as_view(), name='un_login'),
        path('logout/', LogoutView.as_view(), name='un_logout'),
    ])

if settings.UNCHAINED_AUTH.get('PWD_REGISTER'):
    from unchained_auth.views.main import RegisterAPIView

    urlpatterns.extend([
        path('register/', RegisterAPIView.as_view(), name='un_register')
    ])

if settings.UNCHAINED_AUTH.get('PHONE'):
    from unchained_auth.views.phone import PhoneLogin

    urlpatterns.extend([
        path('phone/verify/', PhoneLogin.as_view(), name='un_phone_verify')
    ])

if settings.UNCHAINED_AUTH.get('SOCIAL'):
    from unchained_auth.views.social import SocialView

    urlpatterns.extend([
        re_path(r'social/(?P<backend>[^/]+)/auth/',
                SocialView.as_view(), name='un_social_auth')
    ])

if settings.UNCHAINED_AUTH.get('PWD_RESET'):
    pass



from django.db import models

from unchained_utils.v0.base_classes import unBaseModel, LooseForeignKey
from unchained_utils.v0 import g


class Currency(unBaseModel):
    id = models.CharField(max_length=3, primary_key=True)
    name_en = models.CharField(max_length=100)
    name_ar = models.CharField(max_length=100, null=True, blank=True)


class Country(unBaseModel):
    id = models.CharField(max_length=2, primary_key=True)
    name_en = models.CharField(max_length=100)
    name_ar = models.CharField(max_length=100)

    @property
    def name(self):
        return self.name_ar if g.language == "ar" else self.name_en


class Region(unBaseModel):
    id = models.AutoField(primary_key=True)
    key = models.CharField(max_length=100, unique=True)
    country = LooseForeignKey(Country)
    name_en = models.CharField(max_length=100)
    name_ar = models.CharField(max_length=100, null=True, blank=True)

    @property
    def name(self):
        return self.name_ar if g.language == "ar" else self.name_en

    class Meta:
        unique_together = ('key', 'country')



from rest_framework.viewsets import ReadOnlyModelViewSet

from unchained_chat.models import ChatRoom
from unchained_chat.serializers import ChatRoomSerializer


class ChatRoomViewSet(ReadOnlyModelViewSet):
    serializer_class = ChatRoomSerializer

    def get_queryset(self, queryset=None):
        return ChatRoom.objects.filter(users__in=[self.request.user])







from django.urls import path, include
from rest_framework import routers

from unchained_notification.views import NoListingFCMDeviceViewSet, NotificationViewSet


router = routers.DefaultRouter()
router.register(r'devices', NoListingFCMDeviceViewSet)
router.register(r'notifications', NotificationViewSet, basename='notifications')

urlpatterns = [
    path('', include(router.urls)),
]


from rest_framework import serializers
from rest_framework.serializers import ModelSerializer
from unchained_auth.models import User


class UserSerializer(ModelSerializer):
    password = serializers.CharField(write_only=True)

    class Meta:
        model = User
        fields = [
            'id',
            "email",
            "password",
            "fullname"
        ]


class GoogleUIDSerializer(serializers.Serializer):
    # This can be uid when using phone login or google login
    token = serializers.CharField(
        max_length=4096, required=True, trim_whitespace=True)



from django.contrib import admin

from unchained_chat.models import Message


class MessageInline(admin.TabularInline):
    model = Message
    extra = 1

    def get_readonly_fields(self, request, obj=None):
        if obj:  # obj is not None, so this is an edit
            return list(set(
                [field.name for field in self.model._meta.local_fields] +
                [field.name for field in self.model._meta.local_many_to_many]
            ))  # Return a list or tuple of readonly fields' names
        else:  # This is an addition
            return []

    def has_add_permission(self, request, obj):
        return False

    can_delete = False



from channels.generic.websocket import AsyncJsonWebsocketConsumer


class ChatConsumer(AsyncJsonWebsocketConsumer):
    chatroom = None

    async def connect(self):
        from unchained_chat.models import ChatRoom
        chatroom_id = self.scope["url_route"]["kwargs"]["chatroom_id"]

        if chatroom_id == 'test':
            await self.accept()
        else:
            self.chatroom = ChatRoom.objects.get(nom=self.scope["url_route"]["kwargs"]["chatroom_id"])
            self.group_name = 'CHATROOM{}'.format(self.chatroom.nom)

        if self.scope["user"].is_anonymous:
            await self.accept()  # todo await self.close()
            return
        else:
            await self.accept()
            # todo verify if the  user has access to this group

            await self.channel_layer.group_add(self.group_name, self.channel_name)

    async def disconnect(self, close_code):
        if self.chatroom:
            await self.channel_layer.group_discard(self.group_name, self.channel_name)
        await self.close()

    async def receive_json(self, content, **kwargs):
        from unchained_chat.models import Message

        message = content['message']
        username = (self.scope["user"].is_anonymous and 'anonymous') or self.scope["user"].username

        Message.objects.create(
            room=self.chatroom,
            username=username,
            message=message,
        )

        await self.channel_layer.group_send(
            self.group_name,
            {
                'type': 'chat.message',
                'message': message,
                'username': username
            }
        )

    async def chat_message(self, event):
        # message = event['message']

        print(event)

        del event['type']

        await self.send_json(event)



import uuid

from django.contrib.auth.base_user import BaseUserManager
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.db import models

from unchained_utils.v0.base_classes import LowerCaseEmailField


class UserManager(BaseUserManager):
    def create_user(self, email, password=None):
        if not email:
            raise TypeError('Users must have an email address.')
        user = self.model(email=self.normalize_email(email))
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password):
        if not password:
            raise TypeError('Superusers must have a password.')

        user = self.create_user(email, password)
        user.is_active = True
        user.is_superuser = True
        user.is_staff = True
        user.save()

        return user

class User(AbstractUser):
    objects = UserManager()

    id = models.CharField(max_length=36, primary_key=True, default=uuid.uuid4)
    username = models.CharField(
        max_length=150,
        unique=True,
        validators=[UnicodeUsernameValidator()]
    )
    email = LowerCaseEmailField(unique=True, null=True, blank=True )
    phone = models.CharField(max_length=15, unique=True, null=True, blank=True)
    fullname = models.CharField(max_length=100, null=True, blank=True)
    is_active = models.BooleanField(null=True)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    created_at = models.DateTimeField(db_index=True, auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)



    def save(self, *args, **kwargs):
        if not self.username:
            if self.email:
                self.username = self.email
            elif self.phone:
                self.username = f'user_{self.phone}'
            else:
                self.username = self.id
        super().save()

    def __str__(self):
        return f"{self.email or self.username or self.id}"



import firebase_admin
from firebase_admin import auth
from rest_framework import status, generics
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.authtoken.models import Token


from django.conf import settings
from unchained_auth.serializers import GoogleUIDSerializer

try:
    firebase_credentials = firebase_admin.credentials.Certificate(
        settings.GOOGLE_APPLICATION_CREDENTIALS
    )
    firebase_admin.initialize_app(firebase_credentials)
except (FileNotFoundError,TypeError, ValueError):
    pass


class PhoneLogin(generics.CreateAPIView):
    serializer_class = GoogleUIDSerializer
    permission_classes = (AllowAny,)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        assert serializer.is_valid(), "Invalid token"

        uid = serializer.validated_data['uid']
        firebase_user = auth.get_user(uid)

        self.request.user.phone = firebase_user.phone_number
        self.request.user.user.save()
        token, _ = Token.objects.get_or_create(user=self.request.user)
        return Response({
                    'token': token.key,
        }, status=status.HTTP_200_OK)



# Generated by Django 3.1.4 on 2020-12-30 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('unchained_notification', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



import uuid

from django.contrib.auth import get_user_model
from django.db import models
from unchained_utils.v0 import unBaseModel
from unchained_utils.v0.base_classes import LooseManyToManyField, generate_short_code


def generate_chatroom_code():
    return generate_short_code('R', 8)

class ChatRoom(unBaseModel):
    id = models.CharField(max_length=20, primary_key=True, default=generate_chatroom_code, editable=False)
    name = models.CharField(max_length=30, unique=True)
    users = LooseManyToManyField(get_user_model())
    timestamp = models.DateTimeField(auto_now_add=True, db_index=True)

    @property
    def last_message(self):
        try:
            message = self.messages.order_by('-timestamp')[0]
            return {"username": message.username, "message": message.message, "timestamp": message.timestamp}
        except:
            return {"username": "", "message": "Nouvelle conversation", "timestamp": self.timestamp}

    class Meta:
        ordering = ('-timestamp',)

    @property
    def get_messages(self):
        return self.messages.order_by('-timestamp')[:20]

    def save(self, *args, **kwargs):
        force_update = False

        if self.id:
            force_update = True

        super(ChatRoom, self).save(force_update=force_update)

    def __str__(self):
        return "%s (%s)" % (self.name, self.id)


class Message(models.Model):
    room = models.ForeignKey(ChatRoom, on_delete=models.CASCADE, related_name='messages')
    username = models.CharField(max_length=100)
    message = models.TextField()
    timestamp = models.DateTimeField(auto_now_add=True, db_index=True)

    def __str__(self):
        return self.message

    class Meta:
        ordering = ('timestamp',)

    def save(self, *args, **kwargs):
        super(Message, self).save()



# Generated by Django 3.1.4 on 2020-12-30 12:36

from django.db import migrations, models
import django.db.models.deletion
import unchained_utils.v0.base_classes


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('id', models.CharField(max_length=2, primary_key=True, serialize=False)),
                ('name_en', models.CharField(max_length=100)),
                ('name_ar', models.CharField(max_length=100)),
            ],
            options={
                'get_latest_by': 'created_at',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Currency',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('id', models.CharField(max_length=3, primary_key=True, serialize=False)),
                ('name_en', models.CharField(max_length=100)),
                ('name_ar', models.CharField(blank=True, max_length=100, null=True)),
            ],
            options={
                'get_latest_by': 'created_at',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Region',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('key', models.CharField(max_length=100, unique=True)),
                ('name_en', models.CharField(max_length=100)),
                ('name_ar', models.CharField(blank=True, max_length=100, null=True)),
                ('country', unchained_utils.v0.base_classes.LooseForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='unchained_ref.country')),
            ],
            options={
                'unique_together': {('key', 'country')},
            },
        ),
    ]



from rest_framework.serializers import ModelSerializer

from unchained_notification.models import Notification


class NotificationSerializer(ModelSerializer):
    class Meta:
        model = Notification
        fields = '__all__'



from django.contrib.auth import get_user_model
from django.db import models
from fcm_django.models import FCMDevice
from unchained_utils.v0.base_classes import LooseForeignKey, unBaseModel


class Notification(unBaseModel):
    user = LooseForeignKey(get_user_model(), blank=True)
    text = models.CharField(max_length=200)
    title = models.CharField(max_length=200)
    type = models.CharField(max_length=50)
    payload = models.TextField(null=True, blank=True)

    def send_notification(self):
        if self.user.exists():
            devices = FCMDevice.objects.filter(user_id__in=list(self.user.values_list('id', flat=True)),
                                               active=True)
        else:
            devices = FCMDevice.objects.filter(active=True)
        devices.send_message(title=self.title, body=self.text, sound=True,
                             data={"category": self.type, 'payload': self.payload},
                             extra_kwargs={
                                 "actions": [
                                     {
                                         "title": "Accept",
                                         "action": "accept",
                                         "icon": "icons/heart.png"
                                     },
                                     {
                                         "title": "Ignore",
                                         "action": "ignore",
                                         "icon": "icons/cross.png"
                                     }
                                 ]})

    def __str__(self):
        return f"{self.id}"



import os
from setuptools import find_packages, setup



# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    install_requires=['unchained-utils'],
    package_dir={'': 'src'},
    packages=find_packages('src'),
)



# Generated by Django 3.1.4 on 2020-12-29 17:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import unchained_chat.models
import unchained_utils.v0.base_classes


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ChatRoom',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True, db_index=True)),
                ('id', models.CharField(default=unchained_chat.models.generate_chatroom_code, editable=False, max_length=20, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=30, unique=True)),
                ('timestamp', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('users', unchained_utils.v0.base_classes.LooseManyToManyField(db_constraint=False, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-timestamp',),
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=100)),
                ('message', models.TextField()),
                ('timestamp', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('room', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='unchained_chat.chatroom')),
            ],
            options={
                'ordering': ('timestamp',),
            },
        ),
    ]
